# -*- coding: utf-8 -*-
from pathlib import Path
import pytest

from PIL import Image


@pytest.fixture
def test_image(pytester):
    img = Image.new("RGB", (100, 100), "white")
    filename = pytester.path.joinpath("white.png")
    img.save(filename, format="PNG")


def test_image_snapshot_fixture(pytester, test_image):
    pytester.makepyfile(
        """
        from pathlib import Path
        from PIL import Image
        import pytest

        def test_image(image_snapshot):
            image = Image.new('RGB', (100, 100), 'white')
            image_snapshot(image, "white.png")

        def test_different_image(image_snapshot):
            image = Image.new('RGB', (150, 150), 'white')
            with pytest.raises(AssertionError):
                image_snapshot(image, "white.png")

        def test_create_image_if_missing(image_snapshot):
            assert not Path("red.png").exists()
            image = Image.new('RGB', (100, 100), 'red')
            image_snapshot(image, "red.png")
            assert Path("red.png").exists()
    """
    )

    result = pytester.runpytest("-v")

    result.stdout.fnmatch_lines(
        [
            "*::test_image PASSED*",
        ]
    )

    assert result.ret == 0


def test_image_snapshot_fixture_save_diff(pytester, test_image):
    pytester.makepyfile(
        """
        from pathlib import Path
        from PIL import Image
        import pytest

        def test_different_image(image_snapshot):
            image = Image.new('RGB', (150, 150), 'white')
            with pytest.raises(AssertionError):
                image_snapshot(image, "white.png")
    """
    )

    result = pytester.runpytest("--image-snapshot-save-diff")

    result.assert_outcomes(passed=1)
    assert Path("white.diff.png").exists()
    assert Path("white.new.png").exists()


def test_image_snapshot_fixture_fail_on_missing(pytester):
    pytester.makepyfile(
        """
        from pathlib import Path
        from PIL import Image
        import pytest

        def test_fail_if_image_missing(image_snapshot):
            assert not Path("red.png").exists()
            image = Image.new('RGB', (100, 100), 'red')
            image_snapshot(image, "red.png")
    """
    )

    result = pytester.runpytest("--image-snapshot-fail-if-missing")

    result.assert_outcomes(failed=1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import codecs
from setuptools import setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding='utf-8').read()


setup(
    name='pytest-image-snapshot',
    version='0.4.5',
    author='Bojan Mihelac',
    author_email='bojan@informatikamihelac.com',
    maintainer='Bojan Mihelac',
    maintainer_email='bojan@informatikamihelac.com',
    license='MIT',
    url='https://github.com/bmihelac/pytest-image-snapshot',
    description='A pytest plugin for image snapshot management and comparison.',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    py_modules=['pytest_image_snapshot'],
    python_requires='>=3.5',
    install_requires=['pytest>=3.5.0', 'Pillow'],
    extras_require={
        'pixelmatch': ['pixelmatch>=0.3.0']
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Pytest',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
    ],
    entry_points={
        'pytest11': [
            'image-snapshot = pytest_image_snapshot',
        ],
    },
)



from pathlib import Path

import pytest
from PIL import Image, ImageChops
from PIL import __version__ as PIL_VERSION


class ImageMismatchError(AssertionError):
    """Exception raised when images do not match."""


class ImageNotFoundError(AssertionError):
    """Exception raised when snapshot is missing."""


def pytest_addoption(parser):
    parser.addoption(
        "--image-snapshot-update", action="store_true", help="Update image snapshots"
    )
    parser.addoption(
        "--image-snapshot-fail-if-missing",
        action="store_true",
        help="Fail if snapshot is missing, useful in CI",
    )
    parser.addoption(
        "--image-snapshot-save-diff",
        action="store_true",
        help="Save actual image and diff next to the snapshot, useful in CI",
    )


def extend_to_match_size(img1, img2):
    """
    Extend the smaller image to match the size of the larger one.

    :param img1: First PIL Image object
    :param img2: Second PIL Image object
    :return: Tuple of two PIL Image objects with the same size
    """
    max_width = max(img1.width, img2.width)
    max_height = max(img1.height, img2.height)

    def extend_image(img):
        if img.width == max_width and img.height == max_height:
            return img
        new_img = Image.new(img.mode, (max_width, max_height), (0, 0, 0, 0))
        new_img.paste(img, (0, 0))
        return new_img

    return extend_image(img1), extend_image(img2)


def image_diff(img_1, img_2):
    # PIL < 10 doesn not have alpha_only getbbox argument
    if int(PIL_VERSION.split(".")[0]) <= 10:
        diff = ImageChops.difference(img_1.convert("RGB"), img_2.convert("RGB"))
        return diff if diff.getbbox() else None
    diff = ImageChops.difference(img_1, img_2)
    return diff if diff.getbbox(alpha_only=False) else None


def _image_snapshot(
    img,
    img_path,
    threshold=None,
    update_snapshots=False,
    fail_if_missing=False,
    save_diff=False,
    verbose=0,
):
    img_path = Path(img_path)
    if not update_snapshots and img_path.exists():
        src_image = Image.open(img_path)
        img_1, img_2 = extend_to_match_size(img, src_image)
        diff = image_diff(img_1, img_2)
        if diff:
            if threshold:
                try:
                    from pixelmatch.contrib.PIL import pixelmatch
                except ModuleNotFoundError:
                    raise ModuleNotFoundError(
                        "The 'pixelmatch' package is required for tests using the "
                        "'threshold' argument but is not installed. "
                        "Please install it using 'pip install pixelmatch'."
                    )

                if threshold is True:
                    threshold = 0.1
                mismatch = pixelmatch(
                    img_1, img_2, threshold=threshold, fail_fast=True
                )
                if not mismatch:
                    return
            if save_diff:
                diff.save(img_path.with_suffix(".diff" + img_path.suffix))
                img.save(img_path.with_suffix(".new" + img_path.suffix))
            if verbose:
                diff.show(title="diff")
                if verbose > 1:
                    src_image.show(title="original")
                    img.show(title="new")
            verbose_msg = (
                " Use -v or -vv to display diff."
                if not verbose
                else ""
            )
            snapshot_update_msg = " Use --image-snapshot-update to update snapshot."
            raise ImageMismatchError(
                f"Image does not match the snapshot stored in {img_path}."
                f"{verbose_msg}{snapshot_update_msg}"
            )
        else:
            return
    elif fail_if_missing and not img_path.exists():
        raise ImageNotFoundError(f"Snapshot {img_path} not found.")
    img.save(img_path)


@pytest.fixture
def image_snapshot(request):
    config = request.config

    def wrapper(img, img_path, threshold=None):
        return _image_snapshot(
            img,
            img_path,
            threshold=threshold,
            update_snapshots=config.getoption("--image-snapshot-update"),
            fail_if_missing=config.getoption("--image-snapshot-fail-if-missing"),
            save_diff=config.getoption("--image-snapshot-save-diff"),
            verbose=config.option.verbose,
        )
    return wrapper
